@@ -15,7 +15,6 @@
 */
 
 #include <assert.h>
-#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,6 +25,7 @@
 #define GTK2_UI_URI LV2_UI__GtkUI
 #define QT4_UI_URI  LV2_UI__Qt4UI
 #define X11_UI_URI  LV2_UI__X11UI
+#define WIN_UI_URI  LV2_UI_PREFIX "WindowsUI"
 
 SUIL_API
 unsigned
@@ -45,6 +45,8 @@ suil_ui_supported(const char* container_type_uri,
 	               && !strcmp(ui_type_uri, GTK2_UI_URI))
 	           || (!strcmp(container_type_uri, GTK2_UI_URI)
 	               && !strcmp(ui_type_uri, X11_UI_URI))
+	           || (!strcmp(container_type_uri, GTK2_UI_URI)
+	               && !strcmp(ui_type_uri, WIN_UI_URI))
 	           || (!strcmp(container_type_uri, QT4_UI_URI)
 	               && !strcmp(ui_type_uri, X11_UI_URI))) {
 		return SUIL_WRAPPING_EMBEDDED;
@@ -67,16 +69,19 @@ open_wrapper(SuilHost*      host,
 	const char* module_name = NULL;
 	if (!strcmp(container_type_uri, QT4_UI_URI)
 	    && !strcmp(ui_type_uri, GTK2_UI_URI)) {
-		module_name = "libsuil_gtk2_in_qt4";
+		module_name = "suil_gtk2_in_qt4";
 	} else if (!strcmp(container_type_uri, GTK2_UI_URI)
 	           && !strcmp(ui_type_uri, QT4_UI_URI)) {
-		module_name = "libsuil_qt4_in_gtk2";
+		module_name = "suil_qt4_in_gtk2";
 	} else if (!strcmp(container_type_uri, GTK2_UI_URI)
 	           && !strcmp(ui_type_uri, X11_UI_URI)) {
-		module_name = "libsuil_x11_in_gtk2";
+		module_name = "suil_x11_in_gtk2";
+	} else if (!strcmp(container_type_uri, GTK2_UI_URI)
+	           && !strcmp(ui_type_uri, WIN_UI_URI)) {
+		module_name = "suil_win_in_gtk2";
 	} else if (!strcmp(container_type_uri, QT4_UI_URI)
 	           && !strcmp(ui_type_uri, X11_UI_URI)) {
-		module_name = "libsuil_x11_in_qt4";
+		module_name = "suil_x11_in_qt4";
 	}
 
 	if (!module_name) {
@@ -89,19 +94,20 @@ open_wrapper(SuilHost*      host,
 	const char* const mod_dir = env_dir ? env_dir : SUIL_MODULE_DIR;
 
 	const size_t path_len = strlen(mod_dir)
+		+ strlen(SUIL_DIR_SEP SUIL_MODULE_PREFIX SUIL_MODULE_EXT)
 		+ strlen(module_name)
-		+ strlen(SUIL_MODULE_EXT)
 		+ 2;
 
-	char* const path = calloc(path_len, 1);
-	snprintf(path, path_len, "%s%s%s%s",
-	         mod_dir, SUIL_DIR_SEP, module_name, SUIL_MODULE_EXT);
+	char* const path = (char*)calloc(path_len, 1);
+	snprintf(path, path_len, "%s%s%s%s%s",
+	         mod_dir, SUIL_DIR_SEP,
+	         SUIL_MODULE_PREFIX, module_name, SUIL_MODULE_EXT);
 
 	// Open wrap module
 	dlerror();
 	void* lib = dlopen(path, RTLD_NOW);
 	if (!lib) {
-		SUIL_ERRORF("Unable to open wrap module %s\n", path);
+		SUIL_ERRORF("Unable to open wrap module %s (%s)\n", path, dlerror());
 		return NULL;
 	}
 
@@ -165,7 +171,9 @@ suil_instance_new(SuilHost*                 host,
 	const LV2UI_Descriptor* descriptor = NULL;
 	for (uint32_t i = 0; true; ++i) {
 		const LV2UI_Descriptor* ld = df(i);
-		if (!strcmp(ld->URI, ui_uri)) {
+		if (!ld) {
+			break;
+		} else if (!strcmp(ld->URI, ui_uri)) {
 			descriptor = ld;
 			break;
 		}
@@ -178,7 +186,7 @@ suil_instance_new(SuilHost*                 host,
 	}
 
 	// Create SuilInstance
-	SuilInstance* instance = calloc(1, sizeof(struct SuilInstanceImpl));
+	SuilInstance* instance = (SuilInstance*)calloc(1, sizeof(SuilInstance));
 	if (!instance) {
 		SUIL_ERRORF("Failed to allocate memory for <%s> instance\n", ui_uri);
 		dlclose(lib);
@@ -193,30 +201,31 @@ suil_instance_new(SuilHost*                 host,
 	instance->features[0] = NULL;
 
 	// Copy user provided features
-	unsigned n_features = 0;
-	for (; features && features[n_features]; ++n_features) {
-		const LV2_Feature* f = features[n_features];
-		suil_add_feature(&instance->features, n_features, f->URI, f->data);
+	const LV2_Feature* const* fi         = features;
+	unsigned                  n_features = 0;
+	while (fi && *fi) {
+		const LV2_Feature* f = *fi++;
+		suil_add_feature(&instance->features, &n_features, f->URI, f->data);
 	}
 
 	// Add additional features implemented by SuilHost functions
 	if (host->index_func) {
 		instance->port_map.handle     = controller;
 		instance->port_map.port_index = host->index_func;
-		suil_add_feature(&instance->features, n_features++,
+		suil_add_feature(&instance->features, &n_features,
 		                 LV2_UI__portMap, &instance->port_map);
 	}
 	if (host->subscribe_func && host->unsubscribe_func) {
 		instance->port_subscribe.handle      = controller;
 		instance->port_subscribe.subscribe   = host->subscribe_func;
 		instance->port_subscribe.unsubscribe = host->unsubscribe_func;
-		suil_add_feature(&instance->features, n_features++,
+		suil_add_feature(&instance->features, &n_features,
 		                 LV2_UI__portSubscribe, &instance->port_subscribe);
 	}
 	if (host->touch_func) {
 		instance->touch.handle = controller;
 		instance->touch.touch  = host->touch_func;
-		suil_add_feature(&instance->features, n_features++,
+		suil_add_feature(&instance->features, &n_features,
 		                 LV2_UI__touch, &instance->touch);
 	}
 
@@ -286,6 +295,13 @@ suil_instance_free(SuilInstance* instance)
 	}
 }
 
+SUIL_API
+SuilHandle
+suil_instance_get_handle(SuilInstance* instance)
+{
+	return instance->handle;
+}
+
 SUIL_API
 LV2UI_Widget
 suil_instance_get_widget(SuilInstance* instance)
@@ -301,11 +317,13 @@ suil_instance_port_event(SuilInstance* instance,
                          uint32_t      format,
                          const void*   buffer)
 {
-	instance->descriptor->port_event(instance->handle,
-	                                 port_index,
-	                                 buffer_size,
-	                                 format,
-	                                 buffer);
+	if (instance->descriptor->port_event) {
+		instance->descriptor->port_event(instance->handle,
+		                                 port_index,
+		                                 buffer_size,
+		                                 format,
+		                                 buffer);
+	}
 }
 
 SUIL_API
@@ -313,5 +331,8 @@ const void*
 suil_instance_extension_data(SuilInstance* instance,
                              const char*   uri)
 {
-	return instance->descriptor->extension_data(uri);
+	if (instance->descriptor->extension_data) {
+		return instance->descriptor->extension_data(uri);
+	}
+	return NULL;
 }
